/*****************************************************************************
*                                                                            *
*                                 Layout Example                             *
*                                                                            *
******************************************************************************
* Copyright (C) 2004 by Bill Baxter.   All Rights Reserved.                  *
******************************************************************************
* $Id: layout.cpp,v 1.1 2004/02/15 18:59:02 baxter Exp $                    *
*****************************************************************************/
#include "fx.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/****************************************************************************/


// Mini application object
class ButtonWindow : public FXMainWindow {
  FXDECLARE(ButtonWindow)
protected:
  FXComposite*       contents;      // Container for buttons
  FXVerticalFrame*   controls;      // Switchs to set various modes
  FXuint             layoutFlags;
  FXint              nextButtonNum;
  FXButton*          lastButton;
  
  FXint fixvals[4];
  FXint numRowsCols;

protected:
  ButtonWindow(){}
public:
  enum {
    // Messages to change icon/text relationship
    ID_LAYOUT_NORMAL        =FXMainWindow::ID_LAST,
    ID_LAYOUT_SIDE_TOP      ,/// Pack on top side (default)
    ID_LAYOUT_SIDE_BOTTOM   ,/// Pack on bottom side
    ID_LAYOUT_SIDE_LEFT     ,/// Pack on left side
    ID_LAYOUT_SIDE_RIGHT    ,/// Pack on right side
    ID_LAYOUT_LEFT          ,/// Stick on left (default)
    ID_LAYOUT_RIGHT         ,/// Stick on right
    ID_LAYOUT_TOP           ,/// Stick on top (default)
    ID_LAYOUT_BOTTOM        ,/// Stick on bottom
    
    ID_LAYOUT_FILL_COLUMN   ,/// Matrix column is stretchable
    ID_LAYOUT_FILL_ROW      ,/// Matrix row is stretchable
    ID_LAYOUT_CENTER_X      ,/// Center horizontally
    ID_LAYOUT_CENTER_Y      ,/// Center vertically
    ID_LAYOUT_FIX_X         ,/// X fixed
    ID_LAYOUT_FIX_Y         ,/// Y fixed
    ID_LAYOUT_FIX_WIDTH     ,/// Width fixed
    ID_LAYOUT_FIX_HEIGHT    ,/// height fixed
    ID_LAYOUT_MIN_WIDTH     ,/// Minimum width is the default
    ID_LAYOUT_MIN_HEIGHT    ,/// Minimum height is the default
    ID_LAYOUT_FILL_X        ,/// Stretch or shrink horizontally
    ID_LAYOUT_FILL_Y        ,/// Stretch or shrink vertically
    ID_LAYOUT_FILL          ,/// Stretch or shrink in both directions
    ID_LAYOUT_EXPLICIT      ,/// Explicit placement

    //LAYOUT_RESERVED_1  
    //LAYOUT_RESERVED_2  
    
    ID_MATRIX_BY_COLUMNS    ,/// Uniform width
    ID_MATRIX_BY_ROWS       ,/// Uniform width
    ID_PACK_NORMAL          ,/// Default is each its own size
    ID_PACK_UNIFORM_HEIGHT  ,/// Uniform height
    ID_PACK_UNIFORM_WIDTH   ,/// Uniform width

    ID_CONTAINER_COMPOSITE,
    ID_CONTAINER_HORIZONTAL_FRAME,
    ID_CONTAINER_VERTICAL_FRAME,
    ID_CONTAINER_PACKER,
    ID_CONTAINER_MATRIX,

    ID_FIX_X_VAL,
    ID_FIX_Y_VAL,
    ID_FIX_W_VAL,
    ID_FIX_H_VAL,
    ID_MATRIX_ROWS_COLS,

    
    ID_ADD_BUTTON,          
    ID_CLEAR_BUTTON,          
    // Message to quit application
    ID_QUIT
    };
public:
  long onCmdAddButton(FXObject*,FXSelector,void*);
  long onUpdAddButton(FXObject*,FXSelector,void*);

  long onCmdClearButton(FXObject*,FXSelector,void*);
  long onUpdClearButton(FXObject*,FXSelector,void*);

  long onCmdLayoutFlag(FXObject*,FXSelector,void*);
  long onUpdLayoutFlag(FXObject*,FXSelector,void*);

  long onCmdContainerType(FXObject*,FXSelector,void*);
  long onUpdContainerType(FXObject*,FXSelector,void*);

  long onCmdFixVal(FXObject*,FXSelector,void*);
  long onChgFixVal(FXObject*,FXSelector,void*);
  long onUpdFixVal(FXObject*,FXSelector,void*);

  long onCmdMatVal(FXObject*,FXSelector,void*);
  long onChgMatVal(FXObject*,FXSelector,void*);
  long onUpdMatVal(FXObject*,FXSelector,void*);

  long onCmdQuit(FXObject*,FXSelector,void*);
public:
  ButtonWindow(FXApp* a);
  void create();
  };


/*******************************************************************************/


/*******************************************************************************/


// Map
FXDEFMAP(ButtonWindow) ButtonWindowMap[]={

  FXMAPFUNC(SEL_CLOSE,    0,                                 ButtonWindow::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND,  ButtonWindow::ID_QUIT,             ButtonWindow::onCmdQuit),
  
  FXMAPFUNCS(SEL_COMMAND, ButtonWindow::ID_LAYOUT_NORMAL, ButtonWindow::ID_PACK_UNIFORM_WIDTH, ButtonWindow::onCmdLayoutFlag),
  FXMAPFUNCS(SEL_UPDATE,  ButtonWindow::ID_LAYOUT_NORMAL, ButtonWindow::ID_PACK_UNIFORM_WIDTH, ButtonWindow::onUpdLayoutFlag),

  FXMAPFUNCS(SEL_COMMAND, ButtonWindow::ID_CONTAINER_COMPOSITE, ButtonWindow::ID_CONTAINER_MATRIX, ButtonWindow::onCmdContainerType),
  FXMAPFUNCS(SEL_UPDATE,  ButtonWindow::ID_CONTAINER_COMPOSITE, ButtonWindow::ID_CONTAINER_MATRIX, ButtonWindow::onUpdContainerType),

  FXMAPFUNCS(SEL_COMMAND,  ButtonWindow::ID_FIX_X_VAL, ButtonWindow::ID_FIX_H_VAL, ButtonWindow::onChgFixVal),
  FXMAPFUNCS(SEL_CHANGED,  ButtonWindow::ID_FIX_X_VAL, ButtonWindow::ID_FIX_H_VAL, ButtonWindow::onChgFixVal),
  FXMAPFUNCS(SEL_UPDATE,   ButtonWindow::ID_FIX_X_VAL, ButtonWindow::ID_FIX_H_VAL, ButtonWindow::onUpdFixVal),

  FXMAPFUNC(SEL_COMMAND,  ButtonWindow::ID_MATRIX_ROWS_COLS, ButtonWindow::onChgMatVal),
  FXMAPFUNC(SEL_CHANGED,  ButtonWindow::ID_MATRIX_ROWS_COLS, ButtonWindow::onChgMatVal),
  FXMAPFUNC(SEL_UPDATE,   ButtonWindow::ID_MATRIX_ROWS_COLS, ButtonWindow::onUpdMatVal),

  FXMAPFUNC(SEL_COMMAND,  ButtonWindow::ID_ADD_BUTTON,    ButtonWindow::onCmdAddButton),
  FXMAPFUNC(SEL_COMMAND,  ButtonWindow::ID_CLEAR_BUTTON,  ButtonWindow::onCmdClearButton),
  FXMAPFUNC(SEL_UPDATE,   ButtonWindow::ID_CLEAR_BUTTON,  ButtonWindow::onUpdClearButton),
  };


// ButtonApp implementation
FXIMPLEMENT(ButtonWindow,FXMainWindow,ButtonWindowMap,ARRAYNUMBER(ButtonWindowMap))



// Make some windows
ButtonWindow::ButtonWindow(FXApp* a):FXMainWindow(a,"Layout Test",NULL,NULL,DECOR_TITLE|DECOR_MINIMIZE|DECOR_MAXIMIZE|DECOR_CLOSE|DECOR_BORDER|DECOR_STRETCHABLE|DECOR_MENU,100,100){

  //FXBMPIcon *set=new FXBMPIcon(a,settings,0,IMAGE_ALPHAGUESS);

  // Tooltip
  new FXToolTip(getApp());

  layoutFlags = LAYOUT_NORMAL;
  nextButtonNum = 1;
  lastButton = 0;
  fixvals[0] = 30;
  fixvals[1] = 22;
  fixvals[2] = 50;
  fixvals[3] = 80;
  numRowsCols = 3;

  // Controls on right
  controls=new FXVerticalFrame(this,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y);

  // Separator
  new FXVerticalSeparator(this,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_Y|SEPARATOR_GROOVE);

  // Contents
  contents=new FXPacker(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20,20,20);

  new FXLabel(controls,"Widget Layout Styles\tLayout flags for added buttons",NULL,LAYOUT_CENTER_X);
  FXComposite *controlsT=new FXHorizontalFrame(controls,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X);
  FXComposite *controlsB=new FXHorizontalFrame(controls,LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X);
  FXComposite *checksTL=new FXVerticalFrame(controlsT,LAYOUT_SIDE_LEFT|LAYOUT_FILL);
  new FXVerticalSeparator(controlsT);
  FXComposite *checksTR=new FXVerticalFrame(controlsT,LAYOUT_SIDE_RIGHT|LAYOUT_FILL);

  new FXCheckButton(checksTL, "LAYOUT_NORMAL      ", this, ID_LAYOUT_NORMAL        );
  new FXCheckButton(checksTL, "LAYOUT_TOP         ", this, ID_LAYOUT_TOP           );
  new FXCheckButton(checksTL, "LAYOUT_LEFT        ", this, ID_LAYOUT_LEFT          );
  new FXCheckButton(checksTL, "LAYOUT_BOTTOM      ", this, ID_LAYOUT_BOTTOM        );
  new FXCheckButton(checksTL, "LAYOUT_RIGHT       ", this, ID_LAYOUT_RIGHT         );
  new FXCheckButton(checksTL, "LAYOUT_SIDE_TOP    ", this, ID_LAYOUT_SIDE_TOP      );
  new FXCheckButton(checksTL, "LAYOUT_SIDE_LEFT   ", this, ID_LAYOUT_SIDE_LEFT     );
  new FXCheckButton(checksTL, "LAYOUT_SIDE_BOTTOM ", this, ID_LAYOUT_SIDE_BOTTOM   );
  new FXCheckButton(checksTL, "LAYOUT_SIDE_RIGHT  ", this, ID_LAYOUT_SIDE_RIGHT    );
  new FXCheckButton(checksTL, "LAYOUT_CENTER_X    ", this, ID_LAYOUT_CENTER_X      );
  new FXCheckButton(checksTL, "LAYOUT_CENTER_Y    ", this, ID_LAYOUT_CENTER_Y      );
  new FXCheckButton(checksTL, "LAYOUT_FILL_X      ", this, ID_LAYOUT_FILL_X        );
  new FXCheckButton(checksTL, "LAYOUT_FILL_Y      ", this, ID_LAYOUT_FILL_Y        );
  new FXCheckButton(checksTL, "LAYOUT_FILL        ", this, ID_LAYOUT_FILL          );
  new FXCheckButton(checksTL, "LAYOUT_FILL_COLUMN ", this, ID_LAYOUT_FILL_COLUMN   );
  new FXCheckButton(checksTL, "LAYOUT_FILL_ROW    ", this, ID_LAYOUT_FILL_ROW      );

  new FXCheckButton(checksTR, "LAYOUT_MIN_WIDTH   ", this, ID_LAYOUT_MIN_WIDTH     );
  new FXCheckButton(checksTR, "LAYOUT_MIN_HEIGHT  ", this, ID_LAYOUT_MIN_HEIGHT    );
  new FXCheckButton(checksTR, "LAYOUT_FIX_X       ", this, ID_LAYOUT_FIX_X         );
  new FXCheckButton(checksTR, "LAYOUT_FIX_Y       ", this, ID_LAYOUT_FIX_Y         );
  new FXCheckButton(checksTR, "LAYOUT_FIX_WIDTH   ", this, ID_LAYOUT_FIX_WIDTH     );
  new FXCheckButton(checksTR, "LAYOUT_FIX_HEIGHT  ", this, ID_LAYOUT_FIX_HEIGHT    );
  new FXCheckButton(checksTR, "LAYOUT_EXPLICIT    ", this, ID_LAYOUT_EXPLICIT      );
  
  FXMatrix *mat = new FXMatrix(checksTR, 3, MATRIX_BY_COLUMNS);
  FXDial *fd;
  FXint fsft = FRAME_SUNKEN|FRAME_THICK|SPIN_NOMAX;
  new FXLabel(mat, "Fix X:"); new FXSpinner(mat, 10, this, ID_FIX_X_VAL, fsft); 
  fd = new FXDial(mat, this, ID_FIX_X_VAL,DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_CENTER_Y,0,0,23); 
  fd->setRange(0,10000); fd->setNotchSpacing(450);
  new FXLabel(mat, "Fix Y:"); new FXSpinner(mat, 10, this, ID_FIX_Y_VAL, fsft); 
  fd = new FXDial(mat, this, ID_FIX_Y_VAL,LAYOUT_FIX_HEIGHT,0,0,0,23); 
  fd->setRange(0,10000); fd->setNotchSpacing(450);
  new FXLabel(mat, "Fix W:"); new FXSpinner(mat, 10, this, ID_FIX_W_VAL, fsft); 
  fd = new FXDial(mat, this, ID_FIX_W_VAL,DIAL_HORIZONTAL|LAYOUT_FIX_WIDTH|LAYOUT_CENTER_Y,0,0,23); 
  fd->setRange(0,10000); fd->setNotchSpacing(450);
  new FXLabel(mat, "Fix H:"); new FXSpinner(mat, 10, this, ID_FIX_H_VAL, fsft); 
  fd = new FXDial(mat, this, ID_FIX_H_VAL,LAYOUT_FIX_HEIGHT,0,0,0,23); 
  fd->setRange(0,10000); fd->setNotchSpacing(450);

  new FXHorizontalSeparator(controls);
  new FXLabel(controls,"Container Packing Styles\tPacking flags for container",NULL,LAYOUT_CENTER_X);

  FXComposite *ctrlsB = new FXHorizontalFrame(controls,LAYOUT_FILL);
  FXGroupBox *group=new FXGroupBox(ctrlsB,"Container Type",GROUPBOX_TITLE_CENTER|FRAME_RIDGE|LAYOUT_FILL_X);
  new FXRadioButton(group,"FXComposite\tMake left pane an FXComposite",this,ID_CONTAINER_COMPOSITE);
  new FXRadioButton(group,"FXHorizonalFrame\tMake left pane an FXHorizontalFrame",this,ID_CONTAINER_HORIZONTAL_FRAME);
  new FXRadioButton(group,"FXVerticalFrame\tMake left pane an FXVerticalFrame",this,ID_CONTAINER_VERTICAL_FRAME);
  new FXRadioButton(group,"FXPacker\tMake left pane an FXPacker",this,ID_CONTAINER_PACKER);
  new FXRadioButton(group,"FXMatrix\tMake left pane an FXMatrix",this,ID_CONTAINER_MATRIX);

  FXComposite *checksBR=new FXVerticalFrame(ctrlsB,LAYOUT_SIDE_RIGHT|LAYOUT_FILL);
  new FXCheckButton(checksBR, "PACK_NORMAL        ", this, ID_PACK_NORMAL          );
  new FXCheckButton(checksBR, "PACK_UNIFORM_HEIGHT", this, ID_PACK_UNIFORM_HEIGHT  );
  new FXCheckButton(checksBR, "PACK_UNIFORM_WIDTH ", this, ID_PACK_UNIFORM_WIDTH   );
  new FXCheckButton(checksBR, "MATRIX_BY_ROWS     ", this, ID_MATRIX_BY_ROWS       );
  new FXCheckButton(checksBR, "MATRIX_BY_COLUMNS  ", this, ID_MATRIX_BY_COLUMNS    );
  FXComposite *hf = new FXHorizontalFrame(checksBR,LAYOUT_FILL_X);
  new FXLabel(hf, "Matrix Rows/cols:"); new FXSpinner(hf, 4, this, ID_MATRIX_ROWS_COLS, fsft); 

  FXComposite *bframe = new FXHorizontalFrame(controls,LAYOUT_BOTTOM|LAYOUT_FILL_X);
  new FXButton(bframe,"&Add\tAdd a button",NULL,this,ID_ADD_BUTTON,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X);
  new FXButton(bframe,"&Clear\tRemove all buttons",NULL,this,ID_CLEAR_BUTTON,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X);
  new FXButton(bframe,"&Quit\tQuit program",NULL,this,ID_QUIT,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_BOTTOM);
  }


// Start
void ButtonWindow::create(){
  FXMainWindow::create();
  show(PLACEMENT_SCREEN);
  }


// Change the icon/text relationship
long ButtonWindow::onCmdLayoutFlag(FXObject*sender,FXSelector sel,void*){
  FXint v;
  FXbool pack=FALSE;
  sender->handle(this,FXSEL(SEL_COMMAND,ID_GETINTVALUE),&v);
  switch(FXSELID(sel)){
  case ID_LAYOUT_NORMAL:
    if (v) layoutFlags|=LAYOUT_NORMAL;  else layoutFlags&=~LAYOUT_NORMAL; break;
  case ID_LAYOUT_SIDE_TOP:
    if (v) layoutFlags|=LAYOUT_SIDE_TOP; else layoutFlags&=~LAYOUT_SIDE_TOP; break;
  case ID_LAYOUT_SIDE_BOTTOM:
    if (v) layoutFlags|=LAYOUT_SIDE_BOTTOM; else layoutFlags&=~LAYOUT_SIDE_BOTTOM; break;
  case ID_LAYOUT_SIDE_LEFT:
    if (v) layoutFlags|=LAYOUT_SIDE_LEFT; else layoutFlags&=~LAYOUT_SIDE_LEFT; break;
  case ID_LAYOUT_SIDE_RIGHT:
    if (v) layoutFlags|=LAYOUT_SIDE_RIGHT; else layoutFlags&=~LAYOUT_SIDE_RIGHT; break;
  case ID_LAYOUT_LEFT:
    if (v) layoutFlags|=LAYOUT_LEFT; else layoutFlags&=~LAYOUT_LEFT; break;
  case ID_LAYOUT_RIGHT:
    if (v) layoutFlags|=LAYOUT_RIGHT; else layoutFlags&=~LAYOUT_RIGHT; break;
  case ID_LAYOUT_TOP:
    if (v) layoutFlags|=LAYOUT_TOP; else layoutFlags&=~LAYOUT_TOP; break;
  case ID_LAYOUT_BOTTOM:
    if (v) layoutFlags|=LAYOUT_BOTTOM; else layoutFlags&=~LAYOUT_BOTTOM; break;
  case ID_LAYOUT_FILL_COLUMN:
    if (v) layoutFlags|=LAYOUT_FILL_COLUMN; else layoutFlags&=~LAYOUT_FILL_COLUMN; break;
  case ID_LAYOUT_FILL_ROW:
    if (v) layoutFlags|=LAYOUT_FILL_ROW; else layoutFlags&=~LAYOUT_FILL_ROW; break;
  case ID_LAYOUT_CENTER_X:
    if (v) layoutFlags|=LAYOUT_CENTER_X; else layoutFlags&=~LAYOUT_CENTER_X; break;
  case ID_LAYOUT_CENTER_Y:
    if (v) layoutFlags|=LAYOUT_CENTER_Y; else layoutFlags&=~LAYOUT_CENTER_Y; break;
  case ID_LAYOUT_FIX_X:
    if (v) layoutFlags|=LAYOUT_FIX_X; else layoutFlags&=~LAYOUT_FIX_X; break;
  case ID_LAYOUT_FIX_Y:
    if (v) layoutFlags|=LAYOUT_FIX_Y; else layoutFlags&=~LAYOUT_FIX_Y; break;
  case ID_LAYOUT_FIX_WIDTH:
    if (v) layoutFlags|=LAYOUT_FIX_WIDTH; else layoutFlags&=~LAYOUT_FIX_WIDTH; break;
  case ID_LAYOUT_FIX_HEIGHT:
    if (v) layoutFlags|=LAYOUT_FIX_HEIGHT; else layoutFlags&=~LAYOUT_FIX_HEIGHT; break;
  case ID_LAYOUT_MIN_WIDTH:
    if (v) layoutFlags|=LAYOUT_MIN_WIDTH; else layoutFlags&=~LAYOUT_MIN_WIDTH; break;
  case ID_LAYOUT_MIN_HEIGHT:
    if (v) layoutFlags|=LAYOUT_MIN_HEIGHT; else layoutFlags&=~LAYOUT_MIN_HEIGHT; break;
  case ID_LAYOUT_FILL_X:
    if (v) layoutFlags|=LAYOUT_FILL_X; else layoutFlags&=~LAYOUT_FILL_X; break;
  case ID_LAYOUT_FILL_Y:
    if (v) layoutFlags|=LAYOUT_FILL_Y; else layoutFlags&=~LAYOUT_FILL_Y; break;
  case ID_LAYOUT_FILL:
    if (v) layoutFlags|=LAYOUT_FILL; else layoutFlags&=~LAYOUT_FILL; break;
  case ID_LAYOUT_EXPLICIT:
    if (v) layoutFlags|=LAYOUT_EXPLICIT; else layoutFlags&=~LAYOUT_EXPLICIT; break;
  case ID_PACK_NORMAL:
    pack=TRUE;
    if (v) layoutFlags|=PACK_NORMAL; else layoutFlags&=~PACK_NORMAL; break;
  case ID_PACK_UNIFORM_HEIGHT:
    pack=TRUE;
    if (v) layoutFlags|=PACK_UNIFORM_HEIGHT; else layoutFlags&=~PACK_UNIFORM_HEIGHT; break;
  case ID_PACK_UNIFORM_WIDTH:
    pack=TRUE;
    if (v) layoutFlags|=PACK_UNIFORM_WIDTH; else layoutFlags&=~PACK_UNIFORM_WIDTH; break;
  case ID_MATRIX_BY_COLUMNS:
    pack=TRUE;
    if (v) layoutFlags|=MATRIX_BY_COLUMNS; else layoutFlags&=~MATRIX_BY_COLUMNS; break;
  case ID_MATRIX_BY_ROWS:
    pack=TRUE;
    if (v) layoutFlags|=MATRIX_BY_ROWS; else layoutFlags&=~MATRIX_BY_ROWS; break;
    }
  if (lastButton) lastButton->setLayoutHints(layoutFlags);
  if (pack) {
    if (contents->getMetaClass()->isSubClassOf(&FXPacker::metaClass))
      ((FXPacker*)contents)->setPackingHints(layoutFlags);
    if (contents->getMetaClass()->isSubClassOf(&FXMatrix::metaClass))
      ((FXMatrix*)contents)->setMatrixStyle(layoutFlags);
  }
  return 1;
  }


// Update icon/text relationship radio buttons
long ButtonWindow::onUpdLayoutFlag(FXObject* sender,FXSelector sel,void*){
  FXSelector msg=ID_UNCHECK;
  FXWindow *win=(FXWindow*)sender;
  switch(FXSELID(sel)){
  case ID_LAYOUT_NORMAL:
    if (!LAYOUT_NORMAL) win->disable();
    if((layoutFlags&LAYOUT_NORMAL)==LAYOUT_NORMAL) msg=ID_CHECK; break;
  case ID_LAYOUT_SIDE_TOP:
    if (!LAYOUT_SIDE_TOP) win->disable();
    if((layoutFlags&LAYOUT_SIDE_TOP)==LAYOUT_SIDE_TOP) msg=ID_CHECK; break;
  case ID_LAYOUT_SIDE_BOTTOM:
    if (!LAYOUT_SIDE_BOTTOM) win->disable();
    if((layoutFlags&LAYOUT_SIDE_BOTTOM)==LAYOUT_SIDE_BOTTOM) msg=ID_CHECK; break;
  case ID_LAYOUT_SIDE_LEFT:
    if (!LAYOUT_SIDE_LEFT) win->disable();
    if((layoutFlags&LAYOUT_SIDE_LEFT)==LAYOUT_SIDE_LEFT) msg=ID_CHECK; break;
  case ID_LAYOUT_SIDE_RIGHT:
    if (!LAYOUT_SIDE_RIGHT) win->disable();
    if((layoutFlags&LAYOUT_SIDE_RIGHT)==LAYOUT_SIDE_RIGHT) msg=ID_CHECK; break;
  case ID_LAYOUT_LEFT:
    if (!LAYOUT_LEFT) win->disable();
    if((layoutFlags&LAYOUT_LEFT)==LAYOUT_LEFT) msg=ID_CHECK; break;
  case ID_LAYOUT_RIGHT:
    if (!LAYOUT_RIGHT) win->disable();
    if((layoutFlags&LAYOUT_RIGHT)==LAYOUT_RIGHT) msg=ID_CHECK; break;
  case ID_LAYOUT_TOP:
    if (!LAYOUT_TOP) win->disable();
    if((layoutFlags&LAYOUT_TOP)==LAYOUT_TOP) msg=ID_CHECK; break;
  case ID_LAYOUT_BOTTOM:
    if (!LAYOUT_BOTTOM) win->disable();
    if((layoutFlags&LAYOUT_BOTTOM)==LAYOUT_BOTTOM) msg=ID_CHECK; break;
  case ID_LAYOUT_FILL_COLUMN:
    if (!LAYOUT_FILL_COLUMN) win->disable();
    if((layoutFlags&LAYOUT_FILL_COLUMN)==LAYOUT_FILL_COLUMN) msg=ID_CHECK; break;
  case ID_LAYOUT_FILL_ROW:
    if (!LAYOUT_FILL_ROW) win->disable();
    if((layoutFlags&LAYOUT_FILL_ROW)==LAYOUT_FILL_ROW) msg=ID_CHECK; break;
  case ID_LAYOUT_CENTER_X:
    if (!LAYOUT_CENTER_X) win->disable();
    if((layoutFlags&LAYOUT_CENTER_X)==LAYOUT_CENTER_X) msg=ID_CHECK; break;
  case ID_LAYOUT_CENTER_Y:
    if (!LAYOUT_CENTER_Y) win->disable();
    if((layoutFlags&LAYOUT_CENTER_Y)==LAYOUT_CENTER_Y) msg=ID_CHECK; break;
  case ID_LAYOUT_FIX_X:
    if (!LAYOUT_FIX_X) win->disable();
    if((layoutFlags&LAYOUT_FIX_X)==LAYOUT_FIX_X) msg=ID_CHECK; break;
  case ID_LAYOUT_FIX_Y:
    if (!LAYOUT_FIX_Y) win->disable();
    if((layoutFlags&LAYOUT_FIX_Y)==LAYOUT_FIX_Y) msg=ID_CHECK; break;
  case ID_LAYOUT_FIX_WIDTH:
    if (!LAYOUT_FIX_WIDTH) win->disable();
    if((layoutFlags&LAYOUT_FIX_WIDTH)==LAYOUT_FIX_WIDTH) msg=ID_CHECK; break;
  case ID_LAYOUT_FIX_HEIGHT:
    if (!LAYOUT_FIX_HEIGHT) win->disable();
    if((layoutFlags&LAYOUT_FIX_HEIGHT)==LAYOUT_FIX_HEIGHT) msg=ID_CHECK; break;
  case ID_LAYOUT_MIN_WIDTH:
    if (!LAYOUT_MIN_WIDTH) win->disable();
    if((layoutFlags&LAYOUT_MIN_WIDTH)==LAYOUT_MIN_WIDTH) msg=ID_CHECK; break;
  case ID_LAYOUT_MIN_HEIGHT:
    if (!LAYOUT_MIN_HEIGHT) win->disable();
    if((layoutFlags&LAYOUT_MIN_HEIGHT)==LAYOUT_MIN_HEIGHT) msg=ID_CHECK; break;
  case ID_LAYOUT_FILL_X:
    if (!LAYOUT_FILL_X) win->disable();
    if((layoutFlags&LAYOUT_FILL_X)==LAYOUT_FILL_X) msg=ID_CHECK; break;
  case ID_LAYOUT_FILL_Y:
    if (!LAYOUT_FILL_Y) win->disable();
    if((layoutFlags&LAYOUT_FILL_Y)==LAYOUT_FILL_Y) msg=ID_CHECK; break;
  case ID_LAYOUT_FILL:
    if (!LAYOUT_FILL) win->disable();
    if((layoutFlags&LAYOUT_FILL)==LAYOUT_FILL) msg=ID_CHECK; break;
  case ID_LAYOUT_EXPLICIT:
    if (!LAYOUT_EXPLICIT) win->disable();
    if((layoutFlags&LAYOUT_EXPLICIT)==LAYOUT_EXPLICIT) msg=ID_CHECK; break;
  case ID_PACK_NORMAL:
    if (!PACK_NORMAL) win->disable();
    if((layoutFlags&PACK_NORMAL)==PACK_NORMAL) msg=ID_CHECK; break;
  case ID_PACK_UNIFORM_HEIGHT:
    if (!PACK_UNIFORM_HEIGHT) win->disable();
    if((layoutFlags&PACK_UNIFORM_HEIGHT)==PACK_UNIFORM_HEIGHT) msg=ID_CHECK; break;
  case ID_PACK_UNIFORM_WIDTH:
    if (!PACK_UNIFORM_WIDTH) win->disable();
    if((layoutFlags&PACK_UNIFORM_WIDTH)==PACK_UNIFORM_WIDTH) msg=ID_CHECK; break;
  case ID_MATRIX_BY_COLUMNS:
    if (!MATRIX_BY_COLUMNS) win->disable();
    if((layoutFlags&MATRIX_BY_COLUMNS)==MATRIX_BY_COLUMNS) msg=ID_CHECK; break;
  case ID_MATRIX_BY_ROWS:
    if (!MATRIX_BY_ROWS) win->disable();
    if((layoutFlags&MATRIX_BY_ROWS)==MATRIX_BY_ROWS) msg=ID_CHECK; break;
    }
  sender->handle(this,FXSEL(SEL_COMMAND,msg),NULL);
  
  return 1;
  }

long ButtonWindow::onCmdContainerType(FXObject* sender,FXSelector sel,void*){
  switch(FXSELID(sel)){
  case ID_CONTAINER_COMPOSITE:
    //contents->destroy();
    delete contents;
    contents=new FXComposite(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    contents->create();
    break;
  case ID_CONTAINER_HORIZONTAL_FRAME:
    //contents->destroy();
    delete contents;
    contents=new FXHorizontalFrame(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH,0,0,0,0,20,20,20,20);
    contents->create();
    break;
  case ID_CONTAINER_VERTICAL_FRAME:
    //contents->destroy();
    delete contents;
    contents=new FXVerticalFrame(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
    contents->create();
    break;
  case ID_CONTAINER_PACKER:
    //contents->destroy();
    delete contents;
    contents=new FXPacker(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20,20,20);
    contents->create();
    break;
  case ID_CONTAINER_MATRIX:
    delete contents;
    contents=new FXMatrix(this,3,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20,20,20);
    contents->create();
    break;
  }
  if (contents->getMetaClass()->isSubClassOf(&FXPacker::metaClass))
    ((FXPacker*)contents)->setPackingHints(layoutFlags);
  if (contents->getMetaClass()->isSubClassOf(&FXMatrix::metaClass))
    ((FXMatrix*)contents)->setMatrixStyle(layoutFlags);
  nextButtonNum=0;
  lastButton = 0;
  return 1;
}

long ButtonWindow::onUpdContainerType(FXObject* sender,FXSelector sel,void*){
  const FXMetaClass *mc = contents->getMetaClass();
  switch(FXSELID(sel)) {
  case ID_CONTAINER_COMPOSITE:
    sender->handle(this,FXSEL(SEL_COMMAND, mc==&FXComposite::metaClass?ID_CHECK:ID_UNCHECK),0);
    break;
  case ID_CONTAINER_HORIZONTAL_FRAME:
    sender->handle(this,FXSEL(SEL_COMMAND, mc==&FXHorizontalFrame::metaClass?ID_CHECK:ID_UNCHECK),0);
    break;
  case ID_CONTAINER_VERTICAL_FRAME:
    sender->handle(this,FXSEL(SEL_COMMAND, mc==&FXVerticalFrame::metaClass?ID_CHECK:ID_UNCHECK),0);
    break;
  case ID_CONTAINER_PACKER:
    sender->handle(this,FXSEL(SEL_COMMAND, mc==&FXPacker::metaClass?ID_CHECK:ID_UNCHECK),0);
    break;
  case ID_CONTAINER_MATRIX:
    sender->handle(this,FXSEL(SEL_COMMAND, mc==&FXMatrix::metaClass?ID_CHECK:ID_UNCHECK),0);
    break;
  }
    
  return 1;  
}

long ButtonWindow::onChgFixVal(FXObject*sender,FXSelector sel,void*ptr) {
  switch(FXSELID(sel))
  {
  case ID_FIX_X_VAL:  fixvals[0] = (FXint)ptr;  break;
  case ID_FIX_Y_VAL:  fixvals[1] = (FXint)ptr;  break;
  case ID_FIX_W_VAL:  fixvals[2] = (FXint)ptr;  break;
  case ID_FIX_H_VAL:  fixvals[3] = (FXint)ptr;  break;
  }
  if (lastButton && lastButton->getLayoutHints()&LAYOUT_EXPLICIT) 
  {
    int rect[4]={ 
      lastButton->getX(),lastButton->getY(),
      lastButton->getWidth(),lastButton->getHeight()
    };
    FXuint lh = lastButton->getLayoutHints();
    if (lh & LAYOUT_FIX_X) rect[0] = fixvals[0];
    if (lh & LAYOUT_FIX_Y) rect[1] = fixvals[1];
    if (lh & LAYOUT_FIX_WIDTH) rect[2] = fixvals[2];
    if (lh & LAYOUT_FIX_HEIGHT) rect[3] = fixvals[3];
    lastButton->position(rect[0],rect[1],rect[2],rect[3]);
    contents->layout();
  }
  return 1;
}
long ButtonWindow::onUpdFixVal(FXObject*sender,FXSelector sel,void*ptr) {
  switch(FXSELID(sel))
  {
  case ID_FIX_X_VAL:  sender->handle(this, FXSEL(SEL_COMMAND,ID_SETINTVALUE),&fixvals[0]);  break;
  case ID_FIX_Y_VAL:  sender->handle(this, FXSEL(SEL_COMMAND,ID_SETINTVALUE),&fixvals[1]);  break;
  case ID_FIX_W_VAL:  sender->handle(this, FXSEL(SEL_COMMAND,ID_SETINTVALUE),&fixvals[2]);  break;
  case ID_FIX_H_VAL:  sender->handle(this, FXSEL(SEL_COMMAND,ID_SETINTVALUE),&fixvals[3]);  break;
  }
  return 1;
}

long ButtonWindow::onChgMatVal(FXObject*sender,FXSelector sel,void*ptr) 
{
  numRowsCols = (FXint)ptr;
  if (contents->getMetaClass()->isSubClassOf(&FXMatrix::metaClass)){
    ((FXMatrix*)contents)->setNumRows(numRowsCols);
    ((FXMatrix*)contents)->setNumColumns(numRowsCols);
  }
  return 1;
}
long ButtonWindow::onUpdMatVal(FXObject*sender,FXSelector sel,void*ptr) 
{
  sender->handle(this,FXSEL(SEL_COMMAND,ID_SETINTVALUE),&numRowsCols);
  return 1;
}

// Clear buttons from the container
long ButtonWindow::onCmdClearButton(FXObject*,FXSelector sel,void*){
  const FXMetaClass *mc = contents->getMetaClass();
  FXComposite *nc=0;
  if (mc==&FXComposite::metaClass)
    nc=new FXComposite(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  else if (mc==&FXHorizontalFrame::metaClass)
    nc=new FXHorizontalFrame(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y|PACK_UNIFORM_WIDTH,0,0,0,0,20,20,20,20);
  else if (mc==&FXVerticalFrame::metaClass)
    nc=new FXVerticalFrame(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  else if (mc==&FXPacker::metaClass)
    nc=new FXPacker(this,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20,20,20);
  else if (mc==&FXMatrix::metaClass)
    nc=new FXMatrix(this,3,LAYOUT_SIDE_LEFT|FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,20,20,20);

  if (nc) {
    delete contents;
    contents = nc;
    contents->create();
    lastButton = 0;
    nextButtonNum=0;
  }

  return 1;
  }

long ButtonWindow::onUpdClearButton(FXObject* sender,FXSelector sel,void*){
  sender->handle(this,FXSEL(SEL_COMMAND,lastButton?ID_ENABLE:ID_DISABLE),NULL);
  return 1;
  }

// Add a button to the container
long ButtonWindow::onCmdAddButton(FXObject*,FXSelector sel,void*){
  static const FXchar *titles[]={"Button %d", "Wide Button %d", "Tall\nButton %d"};
  lastButton = new FXButton(contents, FXStringFormat(titles[rand()%3],nextButtonNum), 
    NULL, NULL, 0, layoutFlags|FRAME_THICK|FRAME_RAISED, 
    fixvals[0],fixvals[1],fixvals[2],fixvals[3]);
  lastButton->create();
  nextButtonNum++;
  int w = getWidth();
  resize(w-1,getHeight());
  return 1;
  }

long ButtonWindow::onUpdAddButton(FXObject* sender,FXSelector sel,void*){
  FXSelector updatemessage=FXSEL(SEL_COMMAND,ID_UNCHECK);
  switch(FXSELID(sel)){
    }
 // sender->handle(this,updatemessage,NULL);
  return 1;
  }



// Quit the application
long ButtonWindow::onCmdQuit(FXObject*,FXSelector,void*){
  getApp()->exit(0);
  return 1;
  }

/*******************************************************************************/


// Start the whole thing
int main(int argc,char *argv[]){

  // Make application
  FXApp application("Layout","FoxTest");

  // Open display
  application.init(argc,argv);

  // Main window
  new ButtonWindow(&application);

  // Create app
  application.create();

  // Run
  return application.run();
  }


